import sys
import os
import os.path
import textwrap
from nose import main as nose_main
from optparse import OptionParser

_commands = []

def command(f):
    _commands.append(f)

@command
def test():
    """
    Run amuse-admin test, to perform units tests
    
    Without arguments this command will run the core units tests of
    amuse. All unit tests are run using nose and you can give all
    nose arguments here. Examples:
    
    amuse-admin test -v amuse.test.suite.codes_tests
    amuse-admin test -v amuse.test.suite.ext_tests
    
    """
        
    if len(sys.argv) == 1:
        arguments = list(sys.argv)
        arguments.append('-v')
        arguments.append('amuse.test.suite.core_tests')
        sys.argv = arguments
    
    if not os.path.exists('amuse-tests'):
        os.mkdir('amuse-tests')
        
    os.chdir('amuse-tests')
    try:
        nose_main()
    finally:
        os.chdir('..')
        
@command
def help():
    """
    Usage: amuse-admin help subcommand
    
    Type 'amuse-admin help <subcommand>' for help on a specific subcommand.
    """
    if len(sys.argv) > 1:
        commandname = sys.argv[1]
        for x in _commands:
            if x.__name__ == commandname:
                print(textwrap.dedent(x.__doc__).strip())
    else:
        usage()

def usage():
    """
    Usage: amuse-admin command [options] [args]
    
    Type 'amuse-admin help <subcommand>' for help on a specific subcommand.
    """
    print(textwrap.dedent(usage.__doc__).strip())
    print()
    show_available_subcommands()
    
def show_available_subcommands():
    print("Available subcommands:")
    for x in _commands:
        print("    {0}".format(x.__name__))
    

def main():
    
    if len(sys.argv) == 1:
        usage()
    else:
        commandname = sys.argv[1]
        arguments = list(sys.argv)
        del arguments[1]
        sys.argv = arguments
        
        for x in _commands:
            if x.__name__ == commandname:
                x()
        



if __name__ == "__main__":
    main()





import sys
import os
import signal
import time
import datetime
import nose
import nose.plugins
import traceback
import linecache
import inspect

from nose.plugins.capture import Capture
from nose.core import TestProgram
from nose.plugins.skip import Skip, SkipTest
from nose.plugins.doctests import Doctest
from optparse import OptionParser
from subprocess import call, Popen, PIPE


from . import background_test
from . import project
            
background_test.RunTests.instance = background_test.RunTests()


def number_str(number, singular, plural = None):
    if plural == None:
        plural = singular + 's'
    return str(number) + ' ' + (singular if number == 1 else plural)


def get_svn_credentials():
    if 'RVPW' in os.environ:
        svnpassword = os.environ['RVPW']
    else:
        svnpassword = ''
    if 'RVUSR' in os.environ:
        svnusername = os.environ['RVUSR']
    else:
        svnusername = 'reviewboard'
    return svnpassword, svnusername
        
def _run_the_tests(directory, do_update = False):
    svnpassword, svnusername = get_svn_credentials()
    arguments = ["svn", "--no-auth-cache"]
    if svnpassword:
        arguments.extend(('--password', svnpassword))
    if svnusername:
        arguments.extend(('--username', svnusername))
    arguments.append("update")
    call(arguments)
    call(["make", "clean"])
    call(["make", "all"])
    
    print("start test run")
    report = background_test.RunTests.instance.run_tests(None)
    return report
    
class MakeSVNStatusReport(object):
    from xml.dom import minidom 
    
    def start(self):
        
        svnpassword, svnusername = get_svn_credentials()
        arguments = ["svn", "--no-auth-cache"]
        if svnpassword:
            arguments.extend(('--password', svnpassword))
        if svnusername:
            arguments.extend(('--username', svnusername))
        arguments.extend(('info', '--xml'))
        
        process = Popen(arguments, stdout = PIPE, stderr = PIPE)
        stdoutstring, stderrstring = process.communicate()
        doc = self.minidom.parseString(stdoutstring)
        commit_node = list(doc.getElementsByTagName('commit'))[0]
        revision = commit_node.getAttribute('revision')
        self.result = 'SVN revision: {0}'.format(revision)
        print(self.result)
        return self
        
class MakePlatformReport(object):
    import platform
    import mpi4py.MPI
    
    def start(self):
        w = []
        w.append('<table>')
        items = ['uname', 'system', 'node', 'release', 'version', 'machine', 'processor', 'architecture']
        if self.platform.system() == 'Linux':
            items.append('linux_distribution')
        if self.platform.system() == 'Darwin':
            items.append('mac_ver')
                    
        for x in items:
            self.addRow(w, x, str(getattr(self.platform, x)()))
            
        name_of_the_vendor, version = self.mpi4py.MPI.get_vendor()
        self.addRow(w, 'MPI vendor', str(name_of_the_vendor))
        self.addRow(w, 'MPI version', str(version))
        
        w.append('</table>')
        self.result = ''.join(w)
        return self
    
    def addRow(self, w,  string1, string2):
        w.append('<tr>') 
        w.append('<td>')
        w.append(string1)
        w.append('</td>')
        w.append('<td>')
        w.append(string2)
        w.append('</td>')
        w.append('</tr>') 
        w.append('\n')


    

        
class WriteTestReportOnTestingBlog(object):
    
    def __init__(self, report):
        self.report = report
        self.base_directory = os.path.split(__file__)[0]
        self.remote_directory = 'blogs/testing/entries'
        self.local_directory = os.path.join(self.base_directory, "entries")
        if not os.path.exists(self.local_directory):
            os.makedirs(self.local_directory)
    
    def xtitlestring_for_the_report(self):
        delta_time = self.report.end_time - self.report.start_time;
        title = '';
        title += str(datetime.datetime.fromtimestamp(self.report.start_time));
        title += ' ';
        if report.errors > 0:
            title += 'E' + str(report.errors) + ' ';
        
        if report.failures > 0:
            title += 'F' +  str(report.failures) + ' ';
        
        title += '+' + str(report.tests) + ' ';
        if report.skipped > 0:
            title += 'S' +  str(report.skipped) + ' ';
        
        title += str(delta_time) + 's';
        print(title)
        return title
        
    def titlestring_for_the_report(self):
        w = []                                                                                         
        w.append(number_str(self.report.tests,'test'))                                                        
        w.append(' run, ')                                                                             
        if self.report.failures > 0:                                                                          
            w.append(number_str(self.report.failures,'failure'))                                              
            w.append(' ')                                                                              
        if self.report.errors > 0:                                                                            
            w.append(number_str(self.report.errors,'error'))                                                  
            w.append(' ')                                                                              
        if self.report.errors == 0 and self.report.failures == 0:                                                    
            w.append('all tests passed')                                                               
        w.append(' - ')                                                                                
        delta_t = self.report.end_time - self.report.start_time                                                      
        delta_t = round(delta_t, 3)                                                                    
        w.append(str(delta_t))                                                                         
        w.append(' seconds')                                                                           
        return ''.join(w)  
        
    def start(self):
        time_struct = time.gmtime(self.report.start_time)
        filename = time.strftime("%Y%m%d_%H_%M.txt", time_struct)
        path = os.path.join(self.local_directory, filename)
        self.parts = []
        
        self.parts.append(self.titlestring_for_the_report())
        self.parts.append('\n\n')
        
        self.parts.append('<p>')
        self.parts.append(MakeSVNStatusReport().start().result)
        self.parts.append('</p>')
        self.parts.append('\n\n')
        self.parts.append(MakePlatformReport().start().result)
        self.parts.append('\n\n')
        
        testcases = list(self.report.address_to_report.values())

        testcases.sort(key=lambda x: '' if x.address[0] is None else os.path.basename(x.address[0]))
        
        if self.report.failures > 0:
            self.parts.append('<p>Failed tests:</p>')
            self.parts.append('<ul>')
            for x in testcases:
                if x.failed:
                    self.write_item_on_test(x)
            self.parts.append('</ul>')
            
        if self.report.errors > 0:
            self.parts.append('<p>Errored tests:</p>')
            self.parts.append('<ul>')
            for x in testcases:
                if x.errored:
                    self.write_item_on_test(x)
            self.parts.append('</ul>')
            
        if self.report.skipped > 0:
            self.parts.append('<p>Skipped tests:</p>')
            self.parts.append('<ul>')
            for x in testcases:
                if x.skipped:
                    self.write_item_on_test(x)
            self.parts.append('</ul>')
        
        self.parts.append('<p>Tests run:</p>')
        self.parts.append('<ul>')
        for x in testcases:
            self.write_item_on_test(x)
        self.parts.append('</ul>')
        
        with open(path,"w") as file:
            file.write(''.join(self.parts))
            
        call(["scp", "-i", "forreportkey.id", path, "castle.strw.leidenuniv.nl:"+self.remote_directory])
        
    def write_item_on_test(self, x):
        self.parts.append('<li>')
        self.write_link_to_file_in_svn(x.address[0], x.lineno, x.id)
        self.parts.append(' - ')
        delta_t = x.end_time - x.start_time
        delta_t = round(delta_t, 3)
        self.parts.append(str(delta_t))
        self.parts.append(' seconds')
        self.parts.append('\n')
        if x.failed or x.errored:
            self.write_error_and_traceback(x)
        self.parts.append('</li>')
    
    def write_error_and_traceback(self, x):
        self.parts.append('<ul>')
        self.parts.append('<li>')
        for y in x.error_string:
            self.parts.append(y)
        self.parts.append('</li>')
        self.parts.append('<li>') 
        self.parts.append('<ul>')
        for filename, lineno, name, line in x.traceback:
            self.parts.append('<li>') 
            self.write_link_to_file_in_svn(filename, lineno)
            self.parts.append(' - ')
            self.parts.append(name)
            self.parts.append(' - ')
            self.parts.append(str(line))
            self.parts.append('</li>')
        self.parts.append('</ul>')
        self.parts.append('</li>')
        self.parts.append('</ul>')
        
    def write_link_to_file_in_svn(self, filename, lineno, id = None):
        if filename is None:
            self.parts.append(str(id))
            return
            
        if id is None:
            id = filename + ':' + str(lineno)
            
        filename = filename[len(os.getcwd()):]
        if filename.endswith('.pyc'):
            filename = filename[:-3] + 'py'
        
        self.parts.append('<a href="/trac/browser/trunk'+filename+'#L'+str(lineno)+'">')
        self.parts.append(str(id))
        self.parts.append('</a>')
        

def handler(signum, frame):
    if signum == signal.SIGALRM:
        sys.exit("tests took too much time")
    
if __name__ == '__main__':
    parser = OptionParser()
    
    parser.add_option("-d", "--dir", 
      dest="directory",
      help="run tests in DIRECTORY", 
      metavar="DIRECTORY", 
      default=os.getcwd(),
      type="string")
      
    parser.add_option("-u", "--update", 
      dest="do_update",
      help="update the code before running the tests", 
      default=True,
      action="store_true")
    parser.add_option("", "--no-update", 
      dest="do_update",
      help="do not update the code before running the tests", 
      action="store_false")
      
    (options, args) = parser.parse_args()
    
    signal.signal(signal.SIGALRM, handler)
    signal.alarm(120 * 60) #building and testing must be done in 2 hours
    os.setpgrp()
    try:
        report = _run_the_tests(options.directory) 
        WriteTestReportOnTestingBlog(report).start()
    finally:
        #os.killpg(0, signal.SIGKILL)
        pass



import time
import threading
import traceback
import json
import nose
import sys
import linecache
import inspect
import os.path
import queue as queue
import urllib.parse
from io import StringIO
from http.server import HTTPServer, BaseHTTPRequestHandler
import socketserver as socketserver

from mpi4py import MPI
from nose.plugins.capture import Capture
from nose.plugins.skip import Skip, SkipTest
from nose.core import TestProgram
from multiprocessing import Process, Queue
from optparse import OptionParser
from subprocess import call, Popen, PIPE

EDITOR = None


                
                
osascript_to_open_xcode = """on run argv
 set linenumber to (item 1 of argv) as integer
 set filename_string to item 2 of argv
 set file_to_open to POSIX file filename_string
 tell application "Xcode"
  activate
  set doc_to_edit to (open file_to_open)
  tell doc_to_edit 
   set its selection to item linenumber of paragraph  of it
  end tell
 end tell
end run"""

def open_file(path, lineno = 1):
    global EDITOR
    
    if sys.platform == 'darwin':        
        program = Popen(
            ['osascript', '-', str(lineno), os.path.join(os.getcwd(), path) ], 
            stdin = PIPE, stdout = PIPE, stderr = PIPE)
        out, err = program.communicate(osascript_to_open_xcode)
    else:
        possible_programs = (
            ['geany', path, '+'+str(lineno)],
            ['kate', '-u', '--line',str(lineno),path],
            ['emacs', '+'+str(lineno), path],
            ['nedit-client','-line', str(lineno), path],
        )
        
        for program in possible_programs:
            if program[0] == EDITOR:
                returncode = call(['which', program[0]])
                if returncode == 0:
                    call(program)
                    return 
        
        for program in possible_programs:
            returncode = call(['which', program[0]])
            if returncode == 0:
                call(program)
                return 
        
        call([EDITOR, path])
    
class HandleRequest(BaseHTTPRequestHandler):
   
    
    def do_GET(self):
        self.parsed_path = urllib.parse.urlparse(self.path)
        path = self.parsed_path.path[1:]
        method_name = 'do_' + path
        if hasattr(self, method_name):
            method = getattr(self,method_name)
            string, content_type =  method()
        else:
            if path.endswith(".js"):
                string, content_type = self.javascript_file(path)
            else:
                string, content_type = self.index_file()
        
        self.send_response(200)
        self.send_header("Content-type", content_type)
        self.send_header("Content-Length", str(len(string)))        
        self.end_headers()
        self.wfile.write(string)
    
    def do_long_poll(self):
        self.send_response(200)
        self.send_header("Content-Type", "text/javascript")  
        self.send_header("Transfer-Encoding", "chunked")
        self.send_header("Cache-Control", "no-cache, no-store")
        self.send_header("Pragma", "no-cache")
        self.end_headers()
        while True:
            self.server.tests_finished.wait(10.0)
            if self.server.tests_finished.is_set():
                self.send_chunk('true')
                self.server.tests_finished.clear()
            else:
                self.send_chunk('false')
        self.wfile.write('0\r\n\r\n')
        self.wfile.flush()
                    
    def send_chunk(self, string):
        hex_length = hex(len(string))[2:]
        self.wfile.write('%s \r\n' % hex_length)
        self.wfile.flush()

        self.wfile.write(string)
        self.wfile.write('\r\n')
        self.wfile.flush()
       
    def index_file(self):
        base = os.path.split(__file__)[0]
        filename = os.path.join(base, "realtime_test.html")
        with open(filename, "r") as file:
            contents = file.read()
            return contents, 'text/html'
            
    def javascript_file(self, path):
        base = os.path.split(__file__)[0]
        filename = os.path.join(base, path)
        if not os.path.exists(path):
            return  '', 'text/javascript'
        with open(filename, "r") as file:
            contents = file.read()
            return contents, 'text/javascript'
            
    def log_message(self, format, *args):
        pass
        #sys.stderr.write("%s - - [%s] %s\n" %
        #                 (self.address_string(),
        #                  self.log_date_time_string(),
        #                  format%args))
        
    def do_stop(self):
        thread = threading.Thread(target=self.server.stop)
        thread.daemon = True;
        thread.start()
        return 'null', 'text/javascript'
        
    def do_events(self):
        new_events = self.server.get_all_events_since_previous_query()
        string = json.dumps(new_events)
        content_type = 'text/javascript'
        return string, content_type
        
    def do_open_file(self):
        parameters = urllib.parse.parse_qs(self.parsed_path.query)
        path = parameters['path'][0]
        lineno = int(parameters['lineno'][0])
        open_file(path, lineno)
        string = 'null'
        content_type = 'text/javascript'
        return string, content_type
        

class WebServer(socketserver.ThreadingMixIn, HTTPServer):
    
    def __init__(self, port, request_handler):
        HTTPServer.__init__(self, ('', port), request_handler)
        self.daemon_threads = True
        self.events_queue = queue.Queue()
        
    def start(self):
        self.serve_forever()
        
    def stop(self):
        self.shutdown()
        
    def get_all_events_since_previous_query(self):
        try:
            events = []
            while True:
                events.append(self.events_queue.get(False))
        except queue.Empty:
            pass
            
        return events
            
            

    



#!/usr/bin/env python



import sys
import os.path
import os
import urllib.request, urllib.parse, urllib.error
import subprocess
import shutil

import ssl
if hasattr(ssl, '_create_unverified_context'):
    ssl._create_default_https_context = ssl._create_unverified_context

IS_ON_OSX = sys.platform == 'darwin'

def late(function):
    class LateProperty(object):
        def __init__(self, initializer):
            self.initializer = initializer
        def __get__(self, instance, owner):
            if instance is None:
                return self
            value = self.initializer(instance)
            setattr(instance,self.initializer.__name__,value)
            return value
    return LateProperty(function)

    
        
class InstallPrerequisites(object):
    @late
    def prefix(self):
        return sys.prefix
            
    @late
    def static_prefix(self):
        return os.path.normpath(os.path.join(self.prefix, '..', 'static_libs'))
        
    @late
    def applications(self):
       return [
          #('openssl' , [], '0.9.8k' , 'openssl-', '.tar.gz', 'http://www.openssl.org/source/', self.openssl_build),
          (
            'numpy' ,                  #name to refer by
            [],                        #names of prerequisites (unused)
            '1.8.2' ,                  #version string
            'numpy-', '.tar.gz',       #pre- and postfix for filename
            'https://pypi.python.org/packages/source/n/numpy/', #download url, filename is appended
            self.numpy_build          #method to use for building
          ),
          (
            'nose', 
            [], 
            '1.3.0', 
            'nose-' , '.tar.gz', 
            'https://pypi.python.org/packages/source/n/nose/', 
            self.python_build
          ),
          #~ (
            #~ 'distribute', 
            #~ [], 
            #~ '0.6.49', 
            #~ 'distribute-' , '.tar.gz', 
            #~ 'http://pypi.python.org/packages/source/d/distribute/', 
            #~ self.python_build
          #~ ),
          (
            'hdf' ,
            [],  
            '1.8.14',
            'hdf5-' , '.tar.gz' , 
            'https://support.hdfgroup.org/ftp/HDF5/releases/hdf5-1.8/hdf5-1.8.14/src/',
            #'http://www.hdfgroup.org/ftp/HDF5/current/src/',
            self.hdf5_build
          ) ,
          (
            'h5py', 
            ['hdf'], 
            '2.3.1', 
            'h5py-' , '.tar.gz', 
            'https://pypi.python.org/packages/source/h/h5py/', self.h5py_build
          ) ,
          (
            'docutils', 
            [], 
            '0.9.1', 
            'docutils-','.tar.gz', 
            'https://sourceforge.net/projects/docutils/files/docutils/0.9.1/', 
            self.python_build
          ) ,
          (
            'mpich', 
            [], 
            '3.1.2', 
            'mpich-', '.tar.gz', 
            'https://www.mpich.org/static/tarballs/3.1.2/', 
            self.mpich2_build
          ) ,
          (
            'mpi4py', 
            ['mpich2'], 
            '1.3.1', 
            'mpi4py-', '.tar.gz', 
            'https://bitbucket.org/mpi4py/mpi4py/downloads/', 
            self.python_build
          ) ,
          (
            'zlib' ,                   #name to refer by
            [],                         #names of prerequisites (unused)
            '1.2.8' ,                   #version string
            'zlib-', '.tar.gz',        #pre- and postfix for filename
            'https://sourceforge.net/projects/libpng/files/zlib/1.2.8/', #download url, filename is appended
            self.basic_build             #method to use for building - same as for FFTW should work
          ) ,
          #('openmpi', [], '1.3.3', 'openmpi-', '.tar.gz', 'http://www.open-mpi.org/software/ompi/v1.3/downloads/', self.openmpi_build) ,
          ('setuptools', 
           [], 
           '27.2.0',
           'setuptools-', '.tar.gz',
           'https://pypi.python.org/packages/87/ba/54197971d107bc06f5f3fbdc0d728a7ae0b10cafca46acfddba65a0899d8/', self.python_build) ,
          #http://pypi.python.org/packages/2.6/s/setuptools/setuptools-0.6c11-py2.6.egg#md5=bfa92100bd772d5a213eedd356d64086
          (
            'fftw3' ,                  #name to refer by
            [],                        #names of prerequisites (unused)
            '3.3.3' ,                  #version string
            'fftw-', '.tar.gz',        #pre- and postfix for filename
            'http://www.fftw.org/',    #download url, filename is appended
            self.fftw_build            #method to use for building
          ) ,
          (
            'gsl' ,                    #name to refer by
            [],                        #names of prerequisites (unused)
            '1.16' ,                   #version string
            'gsl-', '.tar.gz',         #pre- and postfix for filename
            'https://ftp.gnu.org/gnu/gsl/', #download url, filename is appended
            self.fftw_build            #method to use for building - same as for FFTW should work
          ) ,
          (
            'cmake' ,                   #name to refer by
            [],                         #names of prerequisites (unused)
            '2.8.12' ,                   #version string
            'cmake-', '.tar.gz',        #pre- and postfix for filename
            'https://www.cmake.org/files/v2.8/', #download url, filename is appended
            self.cmake_build             #method to use for building - same as for FFTW should work
          ) ,
          (
            'freetype' ,                   #name to refer by
            [],                         #names of prerequisites (unused)
            '2.4.12' ,                   #version string
            'freetype-', '.tar.gz',        #pre- and postfix for filename
            'https://download.savannah.gnu.org/releases/freetype/', #download url, filename is appended
            self.freetype_build             #method to use for building - same as for FFTW should work
          ) ,
          (
            'png' ,                   #name to refer by
            [],                         #names of prerequisites (unused)
            '1.6.29' ,                   #version string
            'libpng-', '.tar.gz',        #pre- and postfix for filename
            'https://downloads.sourceforge.net/project/libpng/libpng16/older-releases/1.6.29/', #download url, filename is appended
            self.png_build             #method to use for building - same as for FFTW should work
          ) ,
          (
            'netcdf-c' ,
            ['hdf'],  
            '4.4.1',
            'v' , '.tar.gz' , 
            'https://github.com/Unidata/netcdf-c/archive/',
            self.netcdf_build
          ) ,
          (
            'netcdf-fortran' ,
            ['netcdf-c'],  
            '4.4.4',
            'v' , '.tar.gz' , 
            'https://github.com/Unidata/netcdf-fortran/archive/',
            self.netcdf_fortran_build
          ) ,
          (
            'netcdf4-python' ,
            ['netcdf-c'],  
            '1.2.4rel',
            'v' , '.tar.gz' , 
            'https://github.com/Unidata/netcdf4-python/archive/',
            self.netcdfpython_build
          ) ,
          #(
          #  'tcl' ,                   #name to refer by
          #  [],                         #names of prerequisites (unused)
          #  '8.5.13' ,                   #version string
          #  'tcl', '-src.tar.gz',        #pre- and postfix for filename
          #  'http://downloads.sourceforge.net/project/tcl/Tcl/8.5.13/', #download url, filename is appended
          #  self.tcl_build             #method to use for building - same as for FFTW should work
          #) ,
          #(
          #  'tk' ,                   #name to refer by
          #  [],                         #names of prerequisites (unused)
          #  '8.5.13' ,                   #version string
          #  'tk', '-src.tar.gz',        #pre- and postfix for filename
          #  'http://downloads.sourceforge.net/project/tcl/Tcl/8.5.13/', #download url, filename is appended
          #  self.tk_build             #method to use for building - same as for FFTW should work
          #) ,
          (
            'zmq' ,                   #name to refer by
            [],                         #names of prerequisites (unused)
            '3.2.1' ,                   #version string
            'zeromq-', '-rc2.tar.gz',        #pre- and postfix for filename
            'https://github.com/zeromq/zeromq3-x/releases/download/v3.2.1/', #download url, filename is appended
            self.basic_build             #method to use for building - same as for FFTW should work
          ) ,
          (
            'patchelf' ,
            [],  
            '0.8',
            'patchelf-' , '.tar.gz' , 
            'https://nixos.org/releases/patchelf/patchelf-0.8/',
            self.basic_build
          ) ,          
        ]
        
    @late
    def temp_dir(self):
        return os.path.normpath(os.path.join(self.prefix,'..','install'))
    
    
    @late
    def fortran90_compiler(self):
        if 'FC' in os.environ:
            return os.environ['FC']
        else:
            return self.fortran_compiler
            
    @late
    def fortran77_compiler(self):
        if 'F77' in os.environ:
            return os.environ['F77']
        else:
            return None
    
    
    @late
    def fortran_compiler(self):
        if 'FC' in os.environ:
            return os.environ['FC']
        elif 'FORTRAN' in os.environ:
            return os.environ['FORTRAN']
        elif 'F77' in os.environ:
            return os.environ['F77']
        elif 'FORT' in os.environ:
            return os.environ['FORT']
        else:
            return None
            
    @late
    def use_hydra_process_manager(self):
        return False
        
    def setup_temp_dir(self):
        if not os.path.exists(self.temp_dir):
            os.makedirs(self.temp_dir)
    
    def run_application(self, args, cwd, env = None):
        print("starting " , ' '.join(args))
        process = subprocess.Popen(args, cwd=cwd, env = env)
        returncode = process.wait()
        if returncode != 0:
            commandline = ' '.join(args)
            raise Exception("Error when running <" + commandline + ">")
        print("finished " , ' '.join(args))
    
    def h5py_build(self, path):
        
        self.run_application([sys.executable,'setup.py','build','--hdf5='+self.prefix], cwd=path)
        self.run_application([sys.executable,'setup.py','install'], cwd=path)
        
    def setuptools_install(self, path):
        self.run_application(['sh',], cwd=path)
        
    def hdf5_build(self, path):
        commands = []
        commands.append([
          './configure',
          '--prefix='+self.prefix,
          '--enable-shared', 
          '--enable-production',
          '--without-zlib',
          '--with-pthread=/usr', 
          '--enable-threadsafe',
        ])
        commands.append(['make'])
        commands.append(['make', 'install'])
        for x in commands:
            self.run_application(x, path)
    
    def python_build(self, path):
        self.run_application([sys.executable,'setup.py','build'], cwd=path)
        self.run_application([sys.executable,'setup.py','install'], cwd=path)

    def freetype_build(self,path):
        env = os.environ.copy()
        env['CFLAGS'] = "-I"+ self.prefix + '/include'
        env['CPPFLAGS'] = "-I"+ self.prefix + '/include'
        env['LDFLAGS'] = "-L"+ self.prefix + '/lib'
        self.basic_build(path, env)

    def netcdf_build(self,path):
        env = os.environ.copy()
        env['CFLAGS'] = "-I"+ self.prefix + '/include'
        env['CPPFLAGS'] = "-I"+ self.prefix + '/include'
        env['LDFLAGS'] = "-L"+ self.prefix + '/lib'

        commands = []
        command = [
          './configure',
          '--prefix='+self.prefix,
          '--disable-dap',
          '--enable-shared'
        ]
        commands.append(command)
        commands.append(['make'])
        commands.append(['make', 'install'])
        
        for x in commands:
            self.run_application(x, path, env = env)

    def png_build(self,path):
        env = os.environ.copy()
        env['CFLAGS'] = "-I"+ self.prefix + '/include'
        env['CPPFLAGS'] = "-I"+ self.prefix + '/include'
        env['LDFLAGS'] = "-L"+ self.prefix + '/lib'
        self.basic_build(path, env)

# somehow doesn't work if LDFLAGS set..
    def netcdf_fortran_build(self,path):
        env = os.environ.copy()
        env['CPPFLAGS'] = "-I"+ self.prefix + '/include'
        self.basic_build(path, env)

    def netcdfpython_build(self, path):
        self.run_application([sys.executable,'setup.py','build'], cwd=path)
        self.run_application([sys.executable,'setup.py','install'], cwd=path)
    
    def numpy_build(self, path):
        env = os.environ.copy()
        env['BLAS'] = 'None'
        env['LAPACK'] = 'None'
        env['ATLAS'] = 'None'
	self.run_application(['patch', 'gnu.py', os.path.abspath(os.path.join(os.path.dirname(__file__), 'gnu.py.patch'))], cwd=os.path.join(path,'numpy','distutils','fcompiler'), env=env)
        self.run_application([sys.executable,'setup.py','build'], cwd=path, env=env)
        self.run_application([sys.executable,'setup.py','install'], cwd=path, env=env)
        
    def mercurial_build(self, path):
        self.run_application(['make','install','PREFIX='+self.prefix], cwd=path)
    
    def openmpi_build(self, path):
        commands = []
        commands.append([
          './configure','--prefix='+self.prefix,
          #'--enable-mpi-threads', 
          '--enable-cxx-exceptions',
          '--enable-debug',
          '--enable-orterun-prefix-by-default',
        ])
        commands.append(['make'])
        commands.append(['make', 'install'])
        for x in commands:
            self.run_application(x, path)
            
    def mpich2_build(self, path):
        commands = []
        env = os.environ.copy()
        env['CPP'] = '/usr/bin/cpp'
        command = [
          './configure',
          '--prefix='+self.prefix,
          '--enable-shared',
          '--enable-sharedlibs=gcc',
          '--enable-fc', 
          '--with-python='+sys.executable,
          '--with-device=ch3:sock',
        ]
        if self.use_hydra_process_manager:
            command.append('--with-pm=hydra:mpd:gforker')
        else:
            command.append('--with-pm=gforker:hydra')
        if not self.fortran90_compiler is None:
            command.append('FC=' + self.fortran90_compiler)
        
        commands.append(command)
        commands.append(['make'])
        commands.append(['make', 'install'])
        
        for x in commands:
            self.run_application(x, path)
            
        self.check_mpich2_install(commands, path)
        
    def fftw_build(self, path):
        commands = []
        command = [
          './configure',
          '--prefix='+self.static_prefix,
          '--disable-shared',
          '--enable-threads'
        ]
        commands.append(command)
        commands.append(['make'])
        commands.append(['make', 'install'])
        
        for x in commands:
            self.run_application(x, path)
            
    
    def cmake_build(self, path):
        commands = []
        command = [
          './configure',
          '--prefix='+self.static_prefix
        ]
        commands.append(command)
        commands.append(['make'])
        commands.append(['make', 'install'])
        
        for x in commands:
            self.run_application(x, path)
            
    def basic_build(self, path, env=None):
        commands = []
        command = [
          './configure',
          '--prefix='+self.prefix,
          '--enable-shared'
        ]
        commands.append(command)
        commands.append(['make'])
        commands.append(['make', 'install'])
        
        for x in commands:
            self.run_application(x, path, env = env)
            
    def tcl_build(self, path):
        commands = []
        command = [
          './configure',
          '--prefix='+self.prefix,
          '--enable-shared',
          '--enable-threads'
        ]
        commands.append(command)
        commands.append(['make'])
        commands.append(['make', 'install'])
        
        path = os.path.join(path, 'unix')
        for x in commands:
            self.run_application(x, path)
            
    def tk_build(self, path):            
        commands = []
        command = [
          './configure',
          '--prefix='+self.prefix,
          '--enable-shared',
          '--disable-xss',
          '--enable-threads'
        ]
        commands.append(command)
        commands.append(['make'])
        commands.append(['make', 'install'])
        
        path = os.path.join(path, 'unix')
        for x in commands:
            self.run_application(x, path)
            
        
    def check_mpich2_install(self, commands, path):
        bin_directory = os.path.join(self.prefix, 'bin')
        mpif90_filename = os.path.join(bin_directory, 'mpif90')
        if not os.path.exists(mpif90_filename):
            print("-----------------------------------------------------------------")
            print("MPICH build incomplete, no fortran 90 support")
            print("-----------------------------------------------------------------")
            print("The 'mpif90' command was not build")
            print("This is usually caused by an incompatible C and fortran compiler")
            print("Please set the F90, F77 and CC environment variables")
            print() 
            print("After changing the environment variables,")
            print("you can restart the install with:")
            print()
            print("  ./install.py install mpich2 mpi4py")
            print()
            print("You can rerun the build by hand, using:")
            print() 
            print("  cd", path)
            for command in commands:
                print()
                if len(command) < 3:
                    print(' ', ' '.join(command))
                else:
                    print('   \\\n    '.join(command))
            
            sys.exit(1)
            
            
    def openssl_build(self, path):
        commands = []
        commands.append([
          './config','--prefix='+self.prefix,
          ' --openssldir='+self.prefix+'/openssl',
          '--shared'
        ])
        commands.append(['make'])
        commands.append(['make', 'install'])
        for x in commands:
            self.run_application(x, path)

    def download_apps(self, names, skip):
        for (name, dependencies, version, prefix, suffix, url_prefix, function) in self.applications:
            if names and name not in names:
                continue
            if skip and name in skip:
                continue
            app_file = prefix + version + suffix
            app_dir = prefix + version 
            url = url_prefix + app_file
            temp_app_file = os.path.join(self.temp_dir, app_file)
            if not os.path.exists(temp_app_file):
                print("Downloading ", app_file)
                urllib.request.urlretrieve(url, os.path.join(self.temp_dir, app_file))
                print("...Finished")
                
    
    def list_apps(self, names, skip):
        for (name, dependencies, version, prefix, suffix, url_prefix, function) in self.applications:
            if skip and name in skip:
                continue
            print(name, " - dowloaded from", url_prefix)
                
    def unpack_apps(self, names, skip):
        for (name, dependencies, version, prefix, suffix, url_prefix, function) in self.applications:
            if names and name not in names:
                continue
            if skip and name in skip:
                continue
            app_file = prefix + version + suffix
            app_dir = prefix + version 
            url = url_prefix + app_file
            temp_app_file = os.path.join(self.temp_dir, app_file)
            temp_app_dir = os.path.join(self.temp_dir , app_dir)
            if os.path.exists(temp_app_dir):
                shutil.rmtree(temp_app_dir)
            
            print("Unpacking ", app_file)
            try:
                self.run_application(['tar','-xzf',app_file], cwd=self.temp_dir)
            except:
                print("----------------------------------------------------------")
                print("Could not unpack source file of", name)
                print("----------------------------------------------------------")
                print()
                print("Download location may have changed")
                print("Please download the source file yourself, ")
                print("or contact the AMUSE development team.")
                print("https://github.com/amusecode/amuse/issues")
                print()
                print("To download the file you can update the URL in")
                print("one of the following lines and run the command.")
                print()
                print("curl ", url, "-o", temp_app_file)
                print()
                print("or") 
                print()
                print("wget ", url, "-O", temp_app_file)
                print()
                print("Note: The name of the output file must not be changed (after the -o or -O parameter)")
                sys.exit(1)
            print("...Finished")
            
    def extract_path(self, app_file):
        proc=subprocess.Popen(["tar","tf",app_file], stdout=subprocess.PIPE)
        out,err=proc.communicate()
        out=out.split("\n")
        return os.path.split(out[0])[0]
            
    def build_apps(self, names, skip):
        for (name, dependencies, version, prefix, suffix, url_prefix, function) in self.applications:
            if names and name not in names:
                continue
            if skip and name in skip:
                continue
            app_file = prefix + version + suffix
            app_dir = prefix + version 
            temp_app_dir = self.extract_path(os.path.join(self.temp_dir , app_file) )
            temp_app_dir=os.path.join(self.temp_dir, temp_app_dir)
            if not os.path.exists(temp_app_dir):
                if prefix.endswith('-'):
                    app_dir = prefix[:-1]
                else:
                    app_dir = prefix
                temp_app_dir = os.path.join(self.temp_dir , app_dir)
                if not os.path.exists(temp_app_dir):
                    app_file = prefix + version + suffix
                    if app_file.endswith('.tar.gz'):
                        app_dir = app_file[:-len('.tar.gz')]
                    elif app_file.endswith('.tar.bz2'):
                        app_dir = app_file[:-len('.tar.bz2')]
                    else:
                        app_dir, ignore = os.path.os.path.splitext(app_file)
                        
                    temp_app_dir = os.path.join(self.temp_dir , app_dir)
                    if not os.path.exists(temp_app_dir):
                        print("Package was not correctly unpacked: ", app_file)
                        return
    
            print("Building ", app_file)
            function(temp_app_dir)
            print("...Finished")
            
class InstallPrerequisitesOnOSX(InstallPrerequisites):   
    @late
    def prefix(self):
        path = os.path.split(sys.executable)[0]
        if 'Resources' in path:
            return path[:path.index('Resources')]
        else:
            return sys.prefix
            
    @late
    def temp_dir(self):
        return os.path.normpath(os.path.abspath('install'))
        
    @late
    def static_prefix(self):
        return os.path.normpath(os.path.abspath('static_libs'))
    

    def mpich2_build(self, path):
        
        commands = []
        command = [
          './configure',
          '--prefix='+self.prefix,
          '--enable-fc',
          '--enable-shared',
          '--with-python='+sys.executable,
          '--with-pm=gforker',
          '--enable-sharedlibs=osx-gcc',
          '--with-device=ch3:sock',
        ]
        if self.use_hydra_process_manager:
            command.append('--with-pm=hydra:mpd:gforker')
        else:
            command.append('--with-pm=gforker:hydra')
            
        commands.append(command)
        commands.append(['make'])
        commands.append(['make', 'install'])
        for x in commands:
            self.run_application(x, path)
        
        self.check_mpich2_install(commands, path)
        
    def tcl_build(self, path):
        if True:
            return
            
        commands = []
        command = [
          './configure',
          '--prefix='+self.prefix,
          '--enable-shared',
          '--enable-threads'
        ]
        commands.append(command)
        commands.append(['make'])
        commands.append(['make', 'install'])
        
        path = os.path.join(path, 'unix')
        for x in commands:
            self.run_application(x, path)
            
    def tk_build(self, path):
        if True:
            return
            
        commands = []
        command = [
          './configure',
          '--prefix='+self.prefix,
          '--enable-shared',
          '--disable-xss',
          '--enable-threads'
        ]
        commands.append(command)
        commands.append(['make'])
        commands.append(['make', 'install'])
        
        path = os.path.join(path, 'unix')
        for x in commands:
            self.run_application(x, path)
            
            
     
if IS_ON_OSX:
    INSTALL = InstallPrerequisitesOnOSX()
else:
    INSTALL = InstallPrerequisites()
           
def download(names, skip):
    INSTALL.download_apps(names, skip)

def install(names, skip):
    INSTALL.download_apps(names, skip)
    INSTALL.unpack_apps(names, skip)
    INSTALL.build_apps(names, skip)
    
def list(names, skip):
    INSTALL.list_apps(names, skip)

_commands = {
    'download' : download,
    'install' : install,
    'list' : list,
}

if __name__ == '__main__':
    print("Files are installed in: ", INSTALL.prefix)
    print("Files are downloaded to: ", INSTALL.temp_dir)
    print("")
    
    if INSTALL.fortran90_compiler is None or INSTALL.fortran77_compiler is None:
        print("""No fortran 90 compiler environment variable set.
A FORTRAN 90 compiler is needed for MPI and several modules, 
please set FC and F77 first by (bash, replace gfortran with your preferred
compiler):

export FC=gfortran
export F77=gfortran

or (csh):

setenv FC gfortran 
setenv F77 gfortran 

""")
        sys.exit(1)
    else:
        print("Fortran 90 compiler used will be: ", INSTALL.fortran90_compiler)
        print("Fortran 77 compiler used will be: ", INSTALL.fortran77_compiler)
    
    print("")
    INSTALL.setup_temp_dir()
    do = []
    names = []
    flag = False
    skip = []
    
    for x in sys.argv:
        if x in _commands.keys():
            do.append(x)
            flag = True
        else:
            if x == '--hydra':
                INSTALL.use_hydra_process_manager = True
            if flag:
                if x.startswith('no-'):
                    skip.append(x[3:])
                else:
                    names.append(x)
    for x in do:
        _commands[x](names, skip)
    
    if len(do) == 0:
        print("Usage: install.py download|install|list [package names]")
        print("")
        print("download  download the packages to the download directory")
        print("install   unpack and install the packages to the prefix directory")
        print("")
        print("you can also install download or install individual packages")
        print("please specify a list of packages to install")
        print("")
        print("to install all prerequisites do:")
        print("")
        print("./install.py install")
        print("")
        print("to get a list of all packages:")
        print("")
        print("./install.py list")
        print("")
        print("to install mpich2 with the hydra process manager do:")
        print("")
        print("./install.py --hydra install")
        print("")
        
        
        sys.exit(1)
    

        
